@@ -311,7 +311,7 @@ gboolean handledata(GIOChannel* channel, GIOCondition condition, gpointer datap)
   #else
     int outlen=swr_convert(data->swrctx, cam->frame->data, cam->frame->nb_samples, (const uint8_t**)cam->frame->data, cam->frame->nb_samples);
   #endif
-    camera_playsnd(data, cam, (short*)cam->frame->data[0], outlen);
+    if(outlen>0){camera_playsnd(data, cam, (short*)cam->frame->data[0], outlen);}
 #endif
     return 1;
   }
@@ -351,11 +351,11 @@ gboolean handledata(GIOChannel* channel, GIOCondition condition, gpointer datap)
   if(!gotframe){return 1;}
 
   // Convert to RGB24 format
-  unsigned int bufsize=avpicture_get_size(PIX_FMT_RGB24, cam->frame->width, cam->frame->height);
+  unsigned int bufsize=av_image_get_buffer_size(AV_PIX_FMT_RGB24, cam->frame->width, cam->frame->height, 1);
   unsigned char buf[bufsize];
   cam->dstframe->data[0]=buf;
   cam->dstframe->linesize[0]=cam->frame->width*3;
-  struct SwsContext* swsctx=sws_getContext(cam->frame->width, cam->frame->height, cam->frame->format, cam->frame->width, cam->frame->height, AV_PIX_FMT_RGB24, 0, 0, 0, 0);
+  struct SwsContext* swsctx=sws_getContext(cam->frame->width, cam->frame->height, cam->frame->format, cam->frame->width, cam->frame->height, AV_PIX_FMT_RGB24, SWS_FAST_BILINEAR, 0, 0, 0);
   sws_scale(swsctx, (const uint8_t*const*)cam->frame->data, cam->frame->linesize, 0, cam->frame->height, cam->dstframe->data, cam->dstframe->linesize);
   sws_freeContext(swsctx);
 
@@ -424,17 +424,19 @@ void togglecam(GtkButton* button, struct viddata* data)
     ctx->width=320;
     ctx->height=240;
     cam_resolution(cam, (unsigned int*)&ctx->width, (unsigned int*)&ctx->height);
-    ctx->pix_fmt=PIX_FMT_YUV420P;
+    ctx->pix_fmt=AV_PIX_FMT_YUV420P;
     ctx->time_base.num=1;
     ctx->time_base.den=10;
     avcodec_open2(ctx, data->vencoder, 0);
     AVFrame* frame=av_frame_alloc();
-    frame->format=PIX_FMT_RGB24;
+    frame->format=AV_PIX_FMT_RGB24;
     frame->width=ctx->width;
     frame->height=ctx->height;
     av_image_alloc(frame->data, frame->linesize, ctx->width, ctx->height, frame->format, 1);
     AVPacket packet;
+#ifdef AVPACKET_HAS_BUF
     packet.buf=0;
+#endif
     packet.data=0;
     packet.size=0;
     packet.dts=AV_NOPTS_VALUE;
@@ -447,7 +449,7 @@ void togglecam(GtkButton* button, struct viddata* data)
     dstframe->height=ctx->height;
     av_image_alloc(dstframe->data, dstframe->linesize, ctx->width, ctx->height, ctx->pix_fmt, 1);
 
-    struct SwsContext* swsctx=sws_getContext(frame->width, frame->height, PIX_FMT_RGB24, frame->width, frame->height, AV_PIX_FMT_YUV420P, 0, 0, 0, 0);
+    struct SwsContext* swsctx=sws_getContext(frame->width, frame->height, AV_PIX_FMT_RGB24, frame->width, frame->height, AV_PIX_FMT_YUV420P, SWS_FAST_BILINEAR, 0, 0, 0);
 
     while(1)
     {
@@ -469,7 +471,7 @@ packet.size=0;
       write(campipe[1], &frameinfo, 1);
       write(campipe[1], packet.data, packet.size);
 
-      av_free_packet(&packet);
+      av_packet_unref(&packet);
     }
     sws_freeContext(swsctx);
     _exit(0);
