@@ -1,6 +1,6 @@
 /*
     tc_client-gtk, a graphical user interface for tc_client
-    Copyright (C) 2015  alicia@ion.nu
+    Copyright (C) 2015-2016  alicia@ion.nu
     Copyright (C) 2015  Pamela Hiatt
 
     This program is free software: you can redistribute it and/or modify
@@ -58,12 +58,9 @@
 
 struct viddata
 {
-  GtkWidget* box;
   AVCodec* vdecoder;
   AVCodec* vencoder;
   AVCodec* adecoder;
-  int scalewidth;
-  int scaleheight;
 #ifdef HAVE_AVRESAMPLE
   int audiopipe;
   AVAudioResampleContext* resamplectx;
@@ -87,43 +84,6 @@ char frombuild=0; // Running from the build directory
   PROCESS_INFORMATION coreprocess={.hProcess=0};
 #endif
 
-void updatescaling(struct viddata* data, unsigned int width, unsigned int height)
-{
-// TODO: Move updatescaling into media.c?
-  if(!camcount){return;}
-  if(!width){width=gtk_widget_get_allocated_width(data->box);}
-  if(!height){height=gtk_widget_get_allocated_height(data->box);}
-  data->scalewidth=width/camcount;
-  // 3/4 ratio
-  data->scaleheight=data->scalewidth*3/4;
-  unsigned int i;
-  unsigned int labelsize=0;
-  for(i=0; i<camcount; ++i)
-  {
-    if(gtk_widget_get_allocated_height(cams[i].label)>labelsize)
-      labelsize=gtk_widget_get_allocated_height(cams[i].label);
-  }
-  // Fit by height
-  if(height<data->scaleheight+labelsize)
-  {
-    data->scaleheight=height-labelsize;
-    data->scalewidth=data->scaleheight*4/3;
-  }
-  if(data->scalewidth<8){data->scalewidth=8;}
-  if(data->scaleheight<1){data->scaleheight=1;}
-  // TODO: wrapping and stuff
-  // Rescale current images to fit
-  for(i=0; i<camcount; ++i)
-  {
-    GdkPixbuf* pixbuf=gtk_image_get_pixbuf(GTK_IMAGE(cams[i].cam));
-    if(!pixbuf){continue;}
-    GdkPixbuf* old=pixbuf;
-    pixbuf=gdk_pixbuf_scale_simple(pixbuf, data->scalewidth, data->scaleheight, GDK_INTERP_BILINEAR);
-    gtk_image_set_from_pixbuf(GTK_IMAGE(cams[i].cam), pixbuf);
-    g_object_unref(old);
-  }
-}
-
 void printchat(const char* text, const char* pm)
 {
   GtkAdjustment* scroll;
@@ -131,7 +91,7 @@ void printchat(const char* text, const char* pm)
   struct user* user;
   if(pm && (user=finduser(pm)))
   {
-    pm_open(pm, 0);
+    pm_open(pm, 0, data->scroll);
     scroll=user->pm_scroll;
     buffer=user->pm_buffer;
   }else{
@@ -155,7 +115,7 @@ void printchat_color(const char* text, const char* color, unsigned int offset, c
   struct user* user;
   if(pm && (user=finduser(pm)))
   {
-    pm_open(pm, 0);
+    pm_open(pm, 0, data->scroll);
     scroll=user->pm_scroll;
     buffer=user->pm_buffer;
   }else{
@@ -199,8 +159,7 @@ void printchat_color(const char* text, const char* color, unsigned int offset, c
   if(bottom){autoscroll_after(scroll);}
 }
 
-void freebuffer(guchar* pixels, gpointer data){free(pixels);}
-
+unsigned int cameventsource=0;
 char buf[1024];
 gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer datap)
 {
@@ -213,6 +172,96 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
     if(buf[i]=='\r'||buf[i]=='\n'){break;}
   }
   buf[i]=0;
+  if(!strncmp(buf, "Video: ", 7))
+  {
+    char* sizestr=strchr(&buf[7], ' ');
+    if(!sizestr){return 1;}
+    sizestr[0]=0;
+    // Find the camera representation for the given ID
+    struct camera* cam=camera_find(&buf[7]);
+    unsigned int size=strtoul(&sizestr[1], 0, 0);
+    if(!size){return 1;}
+    // Mostly ignore the first byte (contains frame type (e.g. keyframe etc.) in 4 bits and codec in the other 4)
+    --size;
+    AVPacket pkt;
+    av_init_packet(&pkt);
+    unsigned char databuf[size+4];
+    pkt.data=databuf;
+    unsigned char frameinfo;
+    g_io_channel_read_chars(iochannel, (gchar*)&frameinfo, 1, 0, 0);
+//   printf("Frametype-frame: %x\n", ((unsigned int)frameinfo&0xf0)/16);
+//   printf("Frametype-codec: %x\n", (unsigned int)frameinfo&0xf);
+    unsigned int pos=0;
+    while(pos<size)
+    {
+      g_io_channel_read_chars(iochannel, (gchar*)pkt.data+pos, size-pos, &r, 0);
+      pos+=r;
+    }
+    if((frameinfo&0xf)!=2){return 1;} // Not FLV1, get data but discard it
+    if(!cam){printf("No cam found with ID '%s'\n", &buf[7]); return 1;}
+    pkt.size=size;
+    int gotframe;
+    avcodec_decode_video2(cam->vctx, cam->frame, &gotframe, &pkt);
+    if(!gotframe){return 1;}
+
+    if(cam->placeholder) // Remove the placeholder animation if it has it
+    {
+      g_source_remove(cam->placeholder);
+      cam->placeholder=0;
+    }
+    // Scale and convert to RGB24 format
+    unsigned int bufsize=av_image_get_buffer_size(AV_PIX_FMT_RGB24, camsize_scale.width, camsize_scale.height, 1);
+    unsigned char* buf=malloc(bufsize);
+    cam->dstframe->data[0]=buf;
+    cam->dstframe->linesize[0]=camsize_scale.width*3;
+    struct SwsContext* swsctx=sws_getContext(cam->frame->width, cam->frame->height, cam->frame->format, camsize_scale.width, camsize_scale.height, AV_PIX_FMT_RGB24, SWS_BICUBIC, 0, 0, 0);
+    sws_scale(swsctx, (const uint8_t*const*)cam->frame->data, cam->frame->linesize, 0, cam->frame->height, cam->dstframe->data, cam->dstframe->linesize);
+    sws_freeContext(swsctx);
+    camera_postproc(cam, cam->dstframe->data[0], camsize_scale.width, camsize_scale.height);
+
+    GdkPixbuf* oldpixbuf=gtk_image_get_pixbuf(GTK_IMAGE(cam->cam));
+    GdkPixbuf* gdkframe=gdk_pixbuf_new_from_data(cam->dstframe->data[0], GDK_COLORSPACE_RGB, 0, 8, camsize_scale.width, camsize_scale.height, cam->dstframe->linesize[0], freebuffer, 0);
+    gtk_image_set_from_pixbuf(GTK_IMAGE(cam->cam), gdkframe);
+    g_object_unref(oldpixbuf);
+    return 1;
+  }
+  if(!strncmp(buf, "Audio: ", 7))
+  {
+    char* sizestr=strchr(&buf[7], ' ');
+    if(!sizestr){return 1;}
+    sizestr[0]=0;
+    unsigned int size=strtoul(&sizestr[1], 0, 0);
+    if(!size){return 1;}
+    unsigned char frameinfo;
+    g_io_channel_read_chars(iochannel, (gchar*)&frameinfo, 1, 0, 0);
+    --size; // For the byte we read above
+    AVPacket pkt;
+    av_init_packet(&pkt);
+    unsigned char databuf[size];
+    pkt.data=databuf;
+    pkt.size=size;
+    unsigned int pos=0;
+    while(pos<size)
+    {
+      g_io_channel_read_chars(iochannel, (gchar*)pkt.data+pos, size-pos, &r, 0);
+      pos+=r;
+    }
+#if defined(HAVE_AVRESAMPLE) || defined(HAVE_SWRESAMPLE)
+    // Find the camera representation for the given ID (for decoder context)
+    struct camera* cam=camera_find(&buf[7]);
+    if(!cam){printf("No cam found with ID '%s'\n", &buf[7]); return 1;}
+    int gotframe;
+    avcodec_decode_audio4(cam->actx, cam->frame, &gotframe, &pkt);
+    if(!gotframe){return 1;}
+  #ifdef HAVE_AVRESAMPLE
+    int outlen=avresample_convert(data->resamplectx, cam->frame->data, cam->frame->linesize[0], cam->frame->nb_samples, cam->frame->data, cam->frame->linesize[0], cam->frame->nb_samples);
+  #else
+    int outlen=swr_convert(data->swrctx, cam->frame->data, cam->frame->nb_samples, (const uint8_t**)cam->frame->data, cam->frame->nb_samples);
+  #endif
+    if(outlen>0){camera_playsnd(data->audiopipe, cam, (short*)cam->frame->data[0], outlen);}
+#endif
+    return 1;
+  }
   if(!strncmp(buf, "Currently online: ", 18))
   {
     printchat(buf, 0);
@@ -247,6 +296,7 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
   if(!strncmp(buf, "Currently on cam: ", 18))
   {
     printchat(buf, 0);
+    if(!config_get_bool("autoopencams") && config_get_set("autoopencams")){return 1;}
     char* next=&buf[16];
     while(next)
     {
@@ -447,10 +497,14 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
     return 1;
   }
   // Start a stream when someone cams up
-  if(space && !strcmp(space, " cammed up") && (config_get_bool("autoopencams") || !config_get_set("autoopencams")))
+  if(space && !strcmp(space, " cammed up"))
   {
-    space[0]=0;
-    dprintf(tc_client_in[1], "/opencam %s\n", buf);
+    printchat(buf, 0);
+    if(config_get_bool("autoopencams") || !config_get_set("autoopencams"))
+    {
+      space[0]=0;
+      dprintf(tc_client_in[1], "/opencam %s\n", buf);
+    }
     return 1;
   }
   if(!strncmp(buf, "Starting media stream for ", 26))
@@ -464,11 +518,8 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
     if(!idend){return 1;}
     idend[0]=0;
     camera_removebynick(nick); // Remove any duplicates
-    struct camera* cam=camera_new();
-    cam->frame=av_frame_alloc();
-    cam->dstframe=av_frame_alloc();
-    cam->nick=strdup(nick);
-    cam->id=strdup(id);
+    struct camera* cam=camera_new(nick, id);
+    cam->placeholder=g_timeout_add(100, camplaceholder_update, cam->id);
     cam->vctx=avcodec_alloc_context3(data->vdecoder);
     avcodec_open2(cam->vctx, data->vdecoder, 0);
 #if defined(HAVE_AVRESAMPLE) || defined(HAVE_SWRESAMPLE)
@@ -476,144 +527,54 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
     avcodec_open2(cam->actx, data->adecoder, 0);
     cam->samples=0;
 #endif
-    cam->cam=gtk_image_new();
-    cam->box=gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
-    gtk_box_set_homogeneous(GTK_BOX(cam->box), 0);
-    gtk_box_pack_start(GTK_BOX(cam->box), cam->cam, 0, 0, 0);
-    cam->label=gtk_label_new(cam->nick);
-    gtk_box_pack_start(GTK_BOX(cam->box), cam->label, 0, 0, 0);
-    gtk_box_pack_start(GTK_BOX(data->box), cam->box, 0, 0, 0);
+    updatescaling(0, 0, 1);
     gtk_widget_show_all(cam->box);
-    updatescaling(data, 0, 0);
-    while(gtk_events_pending()){gtk_main_iteration();} // Make sure the label gets its size before we calculate scaling
-    updatescaling(data, 0, 0);
     return 1;
   }
   if(!strcmp(buf, "Starting outgoing media stream"))
   {
-    struct camera* cam=camera_new();
-    cam->frame=av_frame_alloc();
-    cam->dstframe=av_frame_alloc();
-    cam->nick=strdup(nickname);
-    cam->id=strdup("out");
-    cam->vctx=avcodec_alloc_context3(data->vdecoder);
-    avcodec_open2(cam->vctx, data->vdecoder, 0);
+    struct camera* cam=camera_new(nickname, "out");
+    cam->vctx=avcodec_alloc_context3(data->vencoder);
+    cam->vctx->pix_fmt=AV_PIX_FMT_YUV420P;
+    cam->vctx->time_base.num=1;
+    cam->vctx->time_base.den=10;
+    cam->vctx->width=camsize_out.width;
+    cam->vctx->height=camsize_out.height;
+    avcodec_open2(cam->vctx, data->vencoder, 0);
+    cam->frame->data[0]=0;
+    cam->frame->width=0;
+    cam->frame->height=0;
+    cam->dstframe->data[0]=0;
+
     cam->actx=0;
-    cam->cam=gtk_image_new();
-    cam->box=gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
-    gtk_box_pack_start(GTK_BOX(cam->box), cam->cam, 0, 0, 0);
-    cam->label=gtk_label_new(cam->nick);
-    gtk_box_pack_start(GTK_BOX(cam->box), cam->label, 0, 0, 0);
-    gtk_box_pack_start(GTK_BOX(data->box), cam->box, 0, 0, 0);
+    updatescaling(0, 0, 1);
     gtk_widget_show_all(cam->box);
-    updatescaling(data, 0, 0);
-    while(gtk_events_pending()){gtk_main_iteration();} // Make sure the label gets its size before we calculate scaling
-    updatescaling(data, 0, 0);
     return 1;
   }
   if(!strncmp(buf, "VideoEnd: ", 10))
   {
     camera_remove(&buf[10]);
-    updatescaling(data, 0, 0);
     return 1;
   }
-  if(!strncmp(buf, "Audio: ", 7))
+  if(!strncmp(buf, "Room topic: ", 12) ||
+     (space && (!strcmp(space, " is not logged in") || !strncmp(space, " is logged in as ", 17))))
   {
-    char* sizestr=strchr(&buf[7], ' ');
-    if(!sizestr){return 1;}
-    sizestr[0]=0;
-    unsigned int size=strtoul(&sizestr[1], 0, 0);
-    if(!size){return 1;}
-    unsigned char frameinfo;
-    g_io_channel_read_chars(iochannel, (gchar*)&frameinfo, 1, 0, 0);
-    --size; // For the byte we read above
-    AVPacket pkt;
-    av_init_packet(&pkt);
-    unsigned char databuf[size];
-    pkt.data=databuf;
-    pkt.size=size;
-    unsigned int pos=0;
-    while(pos<size)
-    {
-      g_io_channel_read_chars(iochannel, (gchar*)pkt.data+pos, size-pos, &r, 0);
-      pos+=r;
-    }
-#if defined(HAVE_AVRESAMPLE) || defined(HAVE_SWRESAMPLE)
-    // Find the camera representation for the given ID (for decoder context)
-    struct camera* cam=camera_find(&buf[7]);
-    if(!cam){printf("No cam found with ID '%s'\n", &buf[7]); return 1;}
-    int gotframe;
-    avcodec_decode_audio4(cam->actx, cam->frame, &gotframe, &pkt);
-    if(!gotframe){return 1;}
-  #ifdef HAVE_AVRESAMPLE
-    int outlen=avresample_convert(data->resamplectx, cam->frame->data, cam->frame->linesize[0], cam->frame->nb_samples, cam->frame->data, cam->frame->linesize[0], cam->frame->nb_samples);
-  #else
-    int outlen=swr_convert(data->swrctx, cam->frame->data, cam->frame->nb_samples, (const uint8_t**)cam->frame->data, cam->frame->nb_samples);
-  #endif
-    camera_playsnd(data->audiopipe, cam, (short*)cam->frame->data[0], outlen);
-#endif
+    printchat(buf, 0);
     return 1;
   }
-  if(strncmp(buf, "Video: ", 7)){printf("Got '%s'\n", buf); fflush(stdout); return 1;} // Ignore anything else that isn't video
-  char* sizestr=strchr(&buf[7], ' ');
-  if(!sizestr){return 1;}
-  sizestr[0]=0;
-  // Find the camera representation for the given ID
-  struct camera* cam=camera_find(&buf[7]);
-  unsigned int size=strtoul(&sizestr[1], 0, 0);
-  if(!size){return 1;}
-  // Mostly ignore the first byte (contains frame type (e.g. keyframe etc.) in 4 bits and codec in the other 4)
-  --size;
-  AVPacket pkt;
-  av_init_packet(&pkt);
-  unsigned char databuf[size+4];
-  pkt.data=databuf;
-  unsigned char frameinfo;
-  g_io_channel_read_chars(iochannel, (gchar*)&frameinfo, 1, 0, 0);
-// printf("Frametype-frame: %x\n", ((unsigned int)frameinfo&0xf0)/16);
-// printf("Frametype-codec: %x\n", (unsigned int)frameinfo&0xf);
-  unsigned int pos=0;
-  while(pos<size)
+  if(!strcmp(buf, "Server disconnected"))
   {
-    g_io_channel_read_chars(iochannel, (gchar*)pkt.data+pos, size-pos, &r, 0);
-    pos+=r;
-  }
-  unsigned int scalewidth=data->scalewidth;
-  unsigned int scaleheight=data->scaleheight;
-  if(!strcmp(&buf[7], "out"))
-  {
-    if(cam)
+    printchat(buf, 0);
+    if(camproc)
     {
-      dprintf(tc_client_in[1], "/video %i\n", size+1);
-      write(tc_client_in[1], &frameinfo, 1);
-      write(tc_client_in[1], pkt.data, size);
-    }else{
-      cam=&campreview;
-      scalewidth=320;
-      scaleheight=240;
+      g_source_remove(cameventsource);
+      kill(camproc, SIGINT);
+      camproc=0;
     }
+    return 1;
   }
-  if((frameinfo&0xf)!=2){return 1;} // Not FLV1, get data but discard it
-  if(!cam){printf("No cam found with ID '%s'\n", &buf[7]); return 1;}
-  pkt.size=size;
-  int gotframe;
-  avcodec_decode_video2(cam->vctx, cam->frame, &gotframe, &pkt);
-  if(!gotframe){return 1;}
-
-  // Scale and convert to RGB24 format
-  unsigned int bufsize=avpicture_get_size(PIX_FMT_RGB24, scalewidth, scaleheight);
-  unsigned char* buf=malloc(bufsize);
-  cam->dstframe->data[0]=buf;
-  cam->dstframe->linesize[0]=scalewidth*3;
-  struct SwsContext* swsctx=sws_getContext(cam->frame->width, cam->frame->height, cam->frame->format, scalewidth, scaleheight, AV_PIX_FMT_RGB24, 0, 0, 0, 0);
-  sws_scale(swsctx, (const uint8_t*const*)cam->frame->data, cam->frame->linesize, 0, cam->frame->height, cam->dstframe->data, cam->dstframe->linesize);
-  sws_freeContext(swsctx);
-
-  GdkPixbuf* oldpixbuf=gtk_image_get_pixbuf(GTK_IMAGE(cam->cam));
-  GdkPixbuf* gdkframe=gdk_pixbuf_new_from_data(cam->dstframe->data[0], GDK_COLORSPACE_RGB, 0, 8, scalewidth, scaleheight, cam->dstframe->linesize[0], freebuffer, 0);
-  gtk_image_set_from_pixbuf(GTK_IMAGE(cam->cam), gdkframe);
-  g_object_unref(oldpixbuf);
-
+  printf("Got '%s'\n", buf);
+  fflush(stdout);
   return 1;
 }
 
@@ -639,7 +600,6 @@ void audiothread(int fd)
 }
 #endif
 
-unsigned int cameventsource=0;
 void togglecam(GtkCheckMenuItem* item, struct viddata* data)
 {
   if(!gtk_check_menu_item_get_active(item))
@@ -660,15 +620,15 @@ void togglecam(GtkCheckMenuItem* item, struct viddata* data)
   // Start a cam thread for the selected cam
   GtkComboBox* combo=GTK_COMBO_BOX(gtk_builder_get_object(gui, "camselect_combo"));
   GIOChannel* channel=camthread(gtk_combo_box_get_active_id(combo), data->vencoder, 100000);
-  cameventsource=g_io_add_watch(channel, G_IO_IN, handledata, 0);
+  cameventsource=g_io_add_watch(channel, G_IO_IN, cam_encode, 0);
 }
 
 gboolean handleresize(GtkWidget* widget, GdkEventConfigure* event, struct viddata* data)
 {
   char bottom=autoscroll_before(data->scroll);
-  if(event->width!=gtk_widget_get_allocated_width(data->box))
+  if(event->width!=gtk_widget_get_allocated_width(cambox))
   {
-    updatescaling(data, event->width, 0);
+    updatescaling(event->width, 0, 0);
   }
 #ifndef _WIN32 // For some reason scrolling as a response to resizing freezes windows
   if(bottom){autoscroll_after(data->scroll);}
@@ -679,7 +639,7 @@ gboolean handleresize(GtkWidget* widget, GdkEventConfigure* event, struct viddat
 void handleresizepane(GObject* obj, GParamSpec* spec, struct viddata* data)
 {
   char bottom=autoscroll_before(data->scroll);
-  updatescaling(data, 0, gtk_paned_get_position(GTK_PANED(obj)));
+  updatescaling(0, gtk_paned_get_position(GTK_PANED(obj)), 0);
 #ifndef _WIN32
   if(bottom){autoscroll_after(data->scroll);}
 #endif
@@ -741,15 +701,20 @@ gboolean inputkeys(GtkWidget* widget, GdkEventKey* event, void* data)
   return 0;
 }
 
-void sendmessage(GtkEntry* entry, void* data)
+char sendingmsg=0;
+void sendmessage(GtkEntry* entry, void* x)
 {
   const char* msg=gtk_entry_get_text(entry);
+  if(!msg[0]){return;} // Don't send empty lines
+  if(sendingmsg){return;}
+  sendingmsg=1;
   inputhistory_add(msg);
   char* pm=0;
   if(!strncmp(msg, "/pm ", 4))
   {
-    pm_open(&msg[4], 1);
+    pm_open(&msg[4], 1, data->scroll);
     gtk_entry_set_text(entry, "");
+    sendingmsg=0;
     return;
   }
   else if(msg[0]!='/') // If we're in a PM tab, send messages as PMs
@@ -763,6 +728,7 @@ void sendmessage(GtkEntry* entry, void* data)
       if(!user) // Person we were PMing with left
       {
         gtk_entry_set_text(entry, "");
+        sendingmsg=0;
         return;
       }
       pm=strdup(user->nick);
@@ -791,6 +757,7 @@ void sendmessage(GtkEntry* entry, void* data)
      !strncmp(msg, "/topic ", 7))
   {
     gtk_entry_set_text(entry, "");
+    sendingmsg=0;
     return;
   }
   if(!strncmp(msg, "/msg ", 5))
@@ -807,6 +774,7 @@ void sendmessage(GtkEntry* entry, void* data)
         gtk_entry_set_text(entry, "");
         printchat("No such user", 0);
         free(pm);
+        sendingmsg=0;
         return;
       }
     }
@@ -825,6 +793,7 @@ void sendmessage(GtkEntry* entry, void* data)
   printchat_color(text, mycolor, 8, pm);
   gtk_entry_set_text(entry, "");
   free(pm);
+  sendingmsg=0;
 }
 
 void startsession(GtkButton* button, void* x)
@@ -861,8 +830,12 @@ void startsession(GtkButton* button, void* x)
   const char* acc_user=gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(gui, "acc_username")));
   const char* acc_pass=gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(gui, "acc_password")));
 #ifdef _WIN32
-  char cmd[strlen("./tc_client --hexcolors -u    0")+strlen(acc_user)+strlen(channel)+strlen(nick)+strlen(chanpass)];
+  char cmd[strlen("./tc_client --hexcolors --cookies tinychat.cookie -u    0")+strlen(acc_user)+strlen(channel)+strlen(nick)+strlen(chanpass)];
   strcpy(cmd, "./tc_client --hexcolors ");
+  if(config_get_bool("storecookies"))
+  {
+    strcat(cmd, "--cookies tinychat.cookie ");
+  }
   if(acc_user[0])
   {
     strcat(cmd, "-u ");
@@ -886,11 +859,24 @@ void startsession(GtkButton* button, void* x)
     close(tc_client_in[1]);
     dup2(tc_client[1], 1);
     dup2(tc_client_in[0], 0);
-    if(acc_user[0])
+    if(config_get_bool("storecookies"))
     {
-      execlp(TC_CLIENT, TC_CLIENT, "--hexcolors", "-u", acc_user, channel, nick, chanpass, (char*)0);
+      const char* home=getenv("HOME");
+      char filename[strlen(home)+strlen("/.tinychat.cookie0")];
+      sprintf(filename, "%s/.tinychat.cookie", home);
+      if(acc_user[0])
+      {
+        execlp(TC_CLIENT, TC_CLIENT, "--hexcolors", "-u", acc_user, "--cookies", filename, channel, nick, chanpass, (char*)0);
+      }else{
+        execlp(TC_CLIENT, TC_CLIENT, "--hexcolors", "--cookies", filename, channel, nick, chanpass, (char*)0);
+      }
     }else{
-      execlp(TC_CLIENT, TC_CLIENT, "--hexcolors", channel, nick, chanpass, (char*)0);
+      if(acc_user[0])
+      {
+        execlp(TC_CLIENT, TC_CLIENT, "--hexcolors", "-u", acc_user, channel, nick, chanpass, (char*)0);
+      }else{
+        execlp(TC_CLIENT, TC_CLIENT, "--hexcolors", channel, nick, chanpass, (char*)0);
+      }
     }
   }
 #endif
@@ -966,9 +952,7 @@ int main(int argc, char** argv)
   // Set up cam selection and preview
   campreview.cam=GTK_WIDGET(gtk_builder_get_object(gui, "camselect_preview"));
   campreview.frame=av_frame_alloc();
-  campreview.dstframe=av_frame_alloc();
-  campreview.vctx=avcodec_alloc_context3(data->vdecoder);
-  avcodec_open2(campreview.vctx, data->vdecoder, 0);
+  campreview.frame->data[0]=0;
   GtkComboBox* combo=GTK_COMBO_BOX(gtk_builder_get_object(gui, "camselect_combo"));
   g_signal_connect(combo, "changed", G_CALLBACK(camselect_change), data->vencoder);
   // Signals for cancelling
@@ -997,8 +981,10 @@ int main(int argc, char** argv)
 
   item=GTK_WIDGET(gtk_builder_get_object(gui, "menuitem_options_settings"));
   g_signal_connect(item, "activate", G_CALLBACK(showsettings), gui);
-  
-  data->box=GTK_WIDGET(gtk_builder_get_object(gui, "cambox"));
+  item=GTK_WIDGET(gtk_builder_get_object(gui, "menuitem_options_settings2"));
+  g_signal_connect(item, "activate", G_CALLBACK(showsettings), gui);
+
+  cambox=GTK_WIDGET(gtk_builder_get_object(gui, "cambox"));
   userlistwidget=GTK_WIDGET(gtk_builder_get_object(gui, "userlistbox"));
   GtkWidget* chatview=GTK_WIDGET(gtk_builder_get_object(gui, "chatview"));
   data->scroll=gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(gtk_builder_get_object(gui, "chatscroll")));
@@ -1052,6 +1038,23 @@ int main(int argc, char** argv)
   g_signal_connect(item, "switch-page", G_CALLBACK(pm_select), 0);
   // Connect signal for captcha
   g_signal_connect(gtk_builder_get_object(gui, "captcha_done"), "clicked", G_CALLBACK(captcha_done), 0);
+  // Connect signals for camera postprocessing
+  g_signal_connect(gtk_builder_get_object(gui, "cam_menu_colors"), "activate", G_CALLBACK(gui_show_camcolors), 0);
+  g_signal_connect(gtk_builder_get_object(gui, "camcolors_min_brightness"), "value-changed", G_CALLBACK(camcolors_adjust_min), 0);
+  g_signal_connect(gtk_builder_get_object(gui, "camcolors_max_brightness"), "value-changed", G_CALLBACK(camcolors_adjust_max), 0);
+  g_signal_connect(gtk_builder_get_object(gui, "camcolors_auto"), "toggled", G_CALLBACK(camcolors_toggle_auto), 0);
+  g_signal_connect(gtk_builder_get_object(gui, "camcolors_flip_horizontal"), "toggled", G_CALLBACK(camcolors_toggle_flip), 0);
+  g_signal_connect(gtk_builder_get_object(gui, "camcolors_flip_vertical"), "toggled", G_CALLBACK(camcolors_toggle_flip), (void*)1);
+  // Connect signal for hiding cameras
+  g_signal_connect(gtk_builder_get_object(gui, "cam_menu_hide"), "activate", G_CALLBACK(gui_hide_cam), 0);
+  // Load placeholder animation for cameras (no video data yet or mic only)
+  if(frombuild)
+  {
+    camplaceholder=gdk_pixbuf_animation_new_from_file("camplaceholder.gif", 0);
+  }else{
+    camplaceholder=gdk_pixbuf_animation_new_from_file(PREFIX "/share/tc_client/camplaceholder.gif", 0);
+  }
+  camplaceholder_iter=gdk_pixbuf_animation_get_iter(camplaceholder, 0);
   // Populate saved channels
   GtkWidget* startbox=GTK_WIDGET(gtk_builder_get_object(gui, "startbox"));
   int channelcount=config_get_int("channelcount");
