@@ -18,13 +18,15 @@
 #include <string.h>
 #include <stdint.h>
 #include <gtk/gtk.h>
-#include "gui.h"
 #include "config.h"
 #include "logging.h"
 #include "compat.h"
 #include "userlist.h"
+#include "media.h"
+#include "gui.h"
 
 extern void startsession(GtkButton* button, void* x);
+extern int tc_client_in[2];
 GtkBuilder* gui;
 
 char autoscroll_before(GtkAdjustment* scroll)
@@ -71,6 +73,9 @@ void settings_reset(GtkBuilder* gui)
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(option), config_get_bool("camdownonjoin"));
   option=GTK_WIDGET(gtk_builder_get_object(gui, "autoopencams"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(option), config_get_set("autoopencams")?config_get_bool("autoopencams"):1);
+  // Misc/cookies
+  option=GTK_WIDGET(gtk_builder_get_object(gui, "storecookiecheckbox"));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(option), config_get_bool("storecookies"));
 }
 
 void showsettings(GtkMenuItem* item, GtkBuilder* gui)
@@ -109,6 +114,9 @@ void savesettings(GtkButton* button, GtkBuilder* gui)
   config_set("camdownonjoin", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(option))?"True":"False");
   option=GTK_WIDGET(gtk_builder_get_object(gui, "autoopencams"));
   config_set("autoopencams", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(option))?"True":"False");
+  // Misc/cookies
+  option=GTK_WIDGET(gtk_builder_get_object(gui, "storecookiecheckbox"));
+  config_set("storecookies", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(option))?"True":"False");
 
   config_save();
   GtkWidget* settings=GTK_WIDGET(gtk_builder_get_object(gui, "settings"));
@@ -330,7 +338,7 @@ void pm_close(GtkButton* btn, GtkWidget* tab)
   user->pm_highlight=0;
 }
 
-void pm_open(const char* nick, char select)
+void pm_open(const char* nick, char select, GtkAdjustment* scroll)
 {
   struct user* user=finduser(nick);
   if(!user){return;}
@@ -342,6 +350,7 @@ void pm_open(const char* nick, char select)
     gtk_notebook_set_current_page(tabs, num);
     return;
   }
+  char bottom=autoscroll_before(scroll); // If PM tabs (with close buttons) are taller we need to make sure pushing down the chat field doesn't make it stop scrolling
   GtkWidget* textview=gtk_text_view_new();
   user->pm_tab=gtk_scrolled_window_new(0, 0);
   user->pm_buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview));
@@ -368,6 +377,7 @@ void pm_open(const char* nick, char select)
   {
     gtk_notebook_set_current_page(tabs, num);
   }
+  if(bottom){autoscroll_after(scroll);}
 }
 
 void pm_highlight(const char* nick)
@@ -387,11 +397,16 @@ void pm_highlight(const char* nick)
 
 char pm_select(GtkNotebook* tabs, GtkWidget* tab, int num, void* x)
 {
-  struct user* user=user_find_by_tab(tab);
-  if(!user){return 0;}
+  if(num<1){return 0;} // Don't try to unhighlight Main
   // Reset highlighting
-  gtk_label_set_text(GTK_LABEL(user->pm_tablabel), user->nick);
-  user->pm_highlight=0;
+  GtkContainer* box=GTK_CONTAINER(gtk_notebook_get_tab_label(tabs, tab));
+  GList* list=gtk_container_get_children(box);
+  GtkLabel* label=list->data; // Should be the first item
+  g_list_free(list);
+  const char* text=gtk_label_get_text(label);
+  gtk_label_set_text(label, text);
+  struct user* user=user_find_by_tab(tab);
+  if(user){user->pm_highlight=0;}
   return 0;
 }
 
@@ -452,3 +467,88 @@ void fontsize_set(double size)
     buffer_updatesize(userlist[i].pm_buffer);
   }
 }
+
+const char* menu_context_cam=0;
+gboolean gui_show_cam_menu(GtkWidget* widget, GdkEventButton* event, const char* id)
+{
+  if(event->button!=3){return 0;} // Only act on right-click
+  struct camera* cam=camera_find(id);
+  free((void*)menu_context_cam);
+  menu_context_cam=strdup(cam->id);
+  GtkMenu* menu=GTK_MENU(gtk_builder_get_object(gui, "cam_menu"));
+  gtk_menu_popup(menu, 0, 0, 0, 0, event->button, event->time);
+  return 1;
+}
+
+void gui_show_camcolors(GtkMenuItem* menuitem, void* x)
+{
+  struct camera* cam=camera_find(menu_context_cam);
+  if(!cam){return;}
+  gtk_widget_show_all(GTK_WIDGET(gtk_builder_get_object(gui, "cam_colors")));
+  GtkAdjustment* adjustment=GTK_ADJUSTMENT(gtk_builder_get_object(gui, "camcolors_min_brightness"));
+  gtk_adjustment_set_value(adjustment, cam->postproc.min_brightness);
+  adjustment=GTK_ADJUSTMENT(gtk_builder_get_object(gui, "camcolors_max_brightness"));
+  gtk_adjustment_set_value(adjustment, cam->postproc.max_brightness);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gtk_builder_get_object(gui, "camcolors_auto")), cam->postproc.autoadjust);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gtk_builder_get_object(gui, "camcolors_flip_horizontal")), cam->postproc.flip_horizontal);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gtk_builder_get_object(gui, "camcolors_flip_vertical")), cam->postproc.flip_vertical);
+}
+
+void camcolors_adjust_min(GtkAdjustment* adjustment, void* x)
+{
+  GtkAdjustment* other=GTK_ADJUSTMENT(gtk_builder_get_object(gui, "camcolors_max_brightness"));
+  double value=gtk_adjustment_get_value(adjustment);
+  if(value>gtk_adjustment_get_value(other))
+  {
+    gtk_adjustment_set_value(other, value);
+  }
+  if(!menu_context_cam){return;}
+  struct camera* cam=camera_find(menu_context_cam);
+  if(!cam){return;}
+  cam->postproc.min_brightness=value;
+}
+
+void camcolors_adjust_max(GtkAdjustment* adjustment, void* x)
+{
+  GtkAdjustment* other=GTK_ADJUSTMENT(gtk_builder_get_object(gui, "camcolors_min_brightness"));
+  double value=gtk_adjustment_get_value(adjustment);
+  if(value<gtk_adjustment_get_value(other))
+  {
+    gtk_adjustment_set_value(other, value);
+  }
+  if(!menu_context_cam){return;}
+  struct camera* cam=camera_find(menu_context_cam);
+  if(!cam){return;}
+  cam->postproc.max_brightness=value;
+}
+
+void camcolors_toggle_auto(GtkToggleButton* button, void* x)
+{
+  if(!menu_context_cam){return;}
+  struct camera* cam=camera_find(menu_context_cam);
+  if(!cam){return;}
+  cam->postproc.autoadjust=gtk_toggle_button_get_active(button);
+}
+
+void camcolors_toggle_flip(GtkToggleButton* button, void* vertical)
+{
+  if(!menu_context_cam){return;}
+  struct camera* cam=camera_find(menu_context_cam);
+  if(!cam){return;}
+  char v=gtk_toggle_button_get_active(button);
+  if(vertical)
+  {
+    cam->postproc.flip_vertical=v;
+  }else{
+    cam->postproc.flip_horizontal=v;
+  }
+}
+
+void gui_hide_cam(GtkMenuItem* menuitem, void* x)
+{
+  if(!menu_context_cam){return;}
+  struct camera* cam=camera_find(menu_context_cam);
+  if(!cam){return;}
+  dprintf(tc_client_in[1], "/closecam %s\n", cam->nick);
+  camera_remove(menu_context_cam);
+}
