@@ -32,11 +32,18 @@ extern void toggle_logging(GtkToggleButton* button, GtkBuilder* gui);
 extern void toggle_youtubecmd(GtkToggleButton* button, GtkBuilder* gui);
 extern void deletechannel(GtkButton* button, void* x);
 extern void channeldialog(GtkButton* button, struct channelopts* opts);
-extern void pm_open(const char* nick, char select);
+extern void pm_open(const char* nick, char select, GtkAdjustment* scroll);
 extern void pm_highlight(const char* nick);
 extern char pm_select(GtkNotebook* tabs, GtkWidget* tab, int num, void* x);
 extern void buffer_setup_colors(GtkTextBuffer* buffer);
 extern void buffer_updatesize(GtkTextBuffer* buffer);
 extern void fontsize_set(double size);
+extern gboolean gui_show_cam_menu(GtkWidget* widget, GdkEventButton* event, const char* id);
+extern void gui_show_camcolors(GtkMenuItem* menuitem, void* x);
+extern void camcolors_adjust_min(GtkAdjustment* adjustment, void* x);
+extern void camcolors_adjust_max(GtkAdjustment* adjustment, void* x);
+extern void camcolors_toggle_auto(GtkToggleButton* button, void* x);
+extern void camcolors_toggle_flip(GtkToggleButton* button, void* vertical);
+extern void gui_hide_cam(GtkMenuItem* menuitem, void* x);
 
 extern GtkBuilder* gui;
